]> git.cworth.org Git - lmno-server/blobdiff - game.js
Add some autofocus attributes to several forms
[lmno-server] / game.js
diff --git a/game.js b/game.js
index eb78f31ecbe531a38801cbbb9b4a466eb968d8cc..f078c354d44164b4f5ad449b372c3f7cb258e056 100644 (file)
--- a/game.js
+++ b/game.js
@@ -9,6 +9,7 @@ class Player {
     this.name = name;
     this.connections = [connection];
     this.team = no_team;
+    this.active = true;
   }
 
   add_connection(connection) {
@@ -23,7 +24,7 @@ class Player {
 
   /* Returns the number of remaining connections after this one is removed. */
   remove_connection(connection) {
-    this.connections.filter(c => c !== connection);
+    this.connections = this.connections.filter(c => c !== connection);
     return this.connections.length;
   }
 
@@ -35,18 +36,31 @@ class Player {
   info_json() {
     return JSON.stringify({
       id: this.id,
+      active: this.active,
       name: this.name,
-      team: this.team.name
+      team: this.team.name,
+      score: this.score
     });
   }
 }
 
+/* This replacer function allows for JSON.stringify to give results
+ * for objects of various types that are used in game state.
+ */
+function stringify_replacer(key, value) {
+  if (typeof value === 'object' && value instanceof Set) {
+    return [...value];
+  }
+  return value;
+}
+
 /* Base class providing common code for game engine implementations. */
 class Game {
   constructor(id) {
     this.id = id;
     this.players = [];
     this.players_by_session = {};
+    this.active_players = 0;
     this.next_player_id = 1;
     this.teams = [];
     this.state = {
@@ -149,16 +163,34 @@ class Game {
   }
 
   add_player(session, connection) {
+    let nickname = session.nickname;
+
     /* First see if we already have a player object for this session. */
-    const existing = this.players_by_session[session.id];
+    let existing = this.players_by_session[session.id];
+
+    /* Otherwise, see if we can match an inactive player by name. */
+    if (! existing) {
+      existing = this.players.find(player =>
+                                   player.name == nickname &&
+                                   ! player.active);
+    }
+
     if (existing) {
+      if (! existing.active) {
+        /* If we're re-activating a previously idled player, then we
+         * need to alert everyone that this player is now back.
+         */
+        existing.active = true;
+        this.active_players++;
+        this.broadcast_event("player-enter", existing.info_json());
+      }
       existing.add_connection(connection);
+      this.players_by_session[session.id] = existing;
       return existing;
     }
 
     /* No existing player. Add a new one. */
     const id = this.next_player_id;
-    let nickname = session.nickname;
     if (nickname === "")
       nickname = "Guest";
     const nickname_orig = nickname;
@@ -175,16 +207,16 @@ class Game {
 
     /* Broadcast before adding player to list (to avoid announcing the
      * new player to itself). */
-    const player_data = JSON.stringify({ id: player.id, name: player.name });
-    this.broadcast_event("player-enter", player_data);
+    this.broadcast_event("player-enter", player.info_json());
 
     this.players.push(player);
     this.players_by_session[session.id] = player;
+    this.active_players++;
     this.next_player_id++;
 
     /* After adding the player to the list, and if we are already past
      * the first move, assign this player to the first team that
-     * doesn't already have a player aissgned (if any). */
+     * doesn't already have a player assigned (if any). */
     if (! this.first_move) {
       const have_players = Array(this.teams.length).fill(false);
       this.players.forEach(p => {
@@ -203,9 +235,9 @@ class Game {
   remove_player_connection(player, connection) {
     const remaining = player.remove_connection(connection);
     if (remaining === 0) {
+      player.active = false;
+      this.active_players--;
       const player_data = JSON.stringify({ id: player.id });
-      this.players.filter(p => p !== player);
-      delete this.players_by_session[player.session_id];
       this.broadcast_event("player-exit", player_data);
     }
   }
@@ -224,6 +256,14 @@ class Game {
     this.broadcast_string(`event: ${type}\ndata: ${data}\n`);
   }
 
+  broadcast_event_object(type, obj) {
+    this.broadcast_event(type, JSON.stringify(obj));
+  }
+
+  game_state_json() {
+    return JSON.stringify(this.state, stringify_replacer);
+  }
+
   handle_events(request, response) {
     /* These headers will keep the connection open so we can stream events. */
     const headers = {
@@ -251,16 +291,17 @@ class Game {
     /* And the player-info event. */
     response.write(`event: player-info\ndata: ${player.info_json()}\n\n`);
 
-    /* As well as player-enter events for all existing players. */
-    this.players.filter(p => p !== player).forEach(p => {
+    /* As well as player-enter events for all existing, active players. */
+    this.players.filter(
+      p => (p !== player
+            && (p.active || p.score))).forEach(p => {
       response.write(`event: player-enter\ndata: ${p.info_json()}\n\n`);
     });
 
     /* Finally, if this game class has a "state" property, stream that
      * current state to the client. */
     if (this.state) {
-      const state_json = JSON.stringify(this.state);
-      response.write(`event: game-state\ndata: ${state_json}\n\n`);
+      response.write(`event: game-state\ndata: ${this.game_state_json()}\n\n`);
     }
   }